pub mod filters;

#[derive(Clone)]
pub enum FilterState<T> {
    /// T passed filter
    Pass(T),
    /// T was not subject to filter.
    Skip(T),
    /// T did not pass filter
    Drop,
}

impl<T> FilterState<T> {
    pub fn to_type(self) -> Option<T> {
        match self {
            FilterState::Pass(v) | FilterState::Skip(v) => Some(v),
            FilterState::Drop => None
        }
    }

    pub fn is_pass(&self) -> bool {
        if let FilterState::Pass(_) = self {
            true
        } else {
            false
        }
    }
    
    pub fn is_skip(&self) -> bool {
        if let FilterState::Skip(_) = self {
            true
        } else {
            false
        }
    }
    
    pub fn is_drop(&self) -> bool {
        if let FilterState::Drop = self {
            true
        } else {
            false
        }
    }
}

pub trait Filter<T> {
    fn filter(&self, f: FilterState<T>) -> FilterState<T> {
        f
    }
}
